# Synopsis
**RNAscClust** is a pipeline designed to cluster a set of paralogous noncoding RNA (ncRNA) sequences based on 
conserved secondary structure. The input to RNAscClust is a set of multiple sequence alignments, each containing 
a paralog sequence to be clustered that is aligned to a set of orthologous sequences. The aim of RNAscClust is 
to aid the discovery of families and classes of ncRNAs.

# Directory contents

This project is organized in several subdirectories whose content is
described below.

### Config/
Configuration files to set the pipeline paramaters and specify path to the dependencies. Includes default config file and samples to override the default values with custom ones.

### bin/
Contains scripts/binaries which do not need to be compiled.

### lib/
Required Perl libraries.


# Installation
Having the tools mentioned below installed and configured is a necessary step for running the pipeline.
## Dependencies

- Generic tools and libs:
    - Python-2
    - Perl
    - GNU R
    - bzip2
    - GNU time
- Python packages:
    - biopython
    - scikit-learn
    - numpy
    - matplotlib

   Under a Debian or Ubuntu system, installing the following debian/ubuntu packages should satisfy these dependencies:
 
```
python python-biopython  scikit-learn r-base-core bzip2 time python-numpy python-numpy-dev python-scipy libatlas-dev python-matplotlib
```

- NSPDK: RNAscClust contains a precompiled version of NSPDK, the graph kernel package used for the generation of sparse feature vectors, locality sensitive hashing, and candidate cluster extraction. 
This binary should run on most Linux-based systems. Otherwise please recompile it from this [NSPDK source package](ftp://biftp.informatik.uni-freiburg.de/pub/RNAscClust/NSPDK.tar.gz). 
- ViennaRNA Package >= 2.1.9 
- PETFold 2.0
- LocARNA >= 1.7.1 
- Infernal 1.0.2 
- CMfinder 0.2
- RNAz 2.1
- R-scape >= 0.2

For installation instruction of each dependency please refer to its manual. If you had problems on downloading, installing or configuring these tools please contact us.
Alternatively it may be possible to use the pre-compiled binaries available on our server, these binaries should run on most Linux-based platforms:

[ftp://biftp.informatik.uni-freiburg.de/pub/RNAscClust/tools-binaries.tar.gz](ftp://biftp.informatik.uni-freiburg.de/pub/RNAscClust/tools-binaries.tar.gz)

## Configuration

The file `Config/default.config` contains the system-specific configuration (e.g. absolute paths to non-generic dependencies) 
and should be configured according to the installed dependencies' paths after installing RNAscClust.
This default configuration file is always loaded by the RNAscClust pipeline. However an additional custom config file can be passed as an argument to the pipeline. 
This custom config file can, for instance, be used to modify RNAscClust parameters for a customized run of the pipeline.

All parameters specified in the custom config file must also appear in the default config file `Config/default.config` 
however, contrary to the default config, the custom config file does not need to be comprehensive.
Custom parameter values will override the default values. Please see `Config/docker-sge.config` as an example for a custom config file that sets only a subset of the RNAscClust parameters.

Please note that paths in the default configuration `Config/default.config` match the directory structure of the tarball 
containing pre-compiled binaries (see above), therefore an RNAscClust setup using precompiled binaries is expected to imply minimal configuration efforts.

### SGE Grid support
RNAscClust pipeline can run under a SGE grid computing computer cluster system. In this case configuration variables prefixed with "SGE_" should be set according to the host system settings.


# Usage
The python script bin/run_rnascclust.py runs the RNAscClust pipeline for an input directory of alignments to be clustered.

```
Args:  ['bin/run_rnascclust.py', '--help']
usage: run_rnascclust.py [-h] --alignments-dir ALIGNMENTS_DIR --result-path
                         RESULT_PATH [--no-knn-vector] [--fasta-alignments]
                         [--local-config LOCAL_CONFIG] [--GC-config GC_CONFIG]
                         [--sge-parallel] [--host-sge HOST_SGE]
                         [--rep-fasta REP_FASTA]
                         [--representative-keyword REPRESENTATIVE_KEYWORD]
                         [--temp-path TEMP_PATH] [--structure-is-given]
                         [--mfe] [--path-graph] [--path-abstract-graph]
                         [--only-representative]
                         [--max-selection-size MAX_SELECTION_SIZE]
                         [--perform-evaluation] [--only-evaluation]
                         [--threshold-reliability THRESHOLD_RELIABILITY]
                         [--predict-structure] [--predict-alifold]
                         [--keep-temp] [--min-class-size MIN_CLASS_SIZE]
                         [--force] [--skip-graph-encoding] [--prefilter-blast]
                         [--encodeN-covarying]
                         [--covarying-Eval COVARYING_EVAL] [--debug]
                         [-job-id JOB_ID]

optional arguments:
  -h, --help            show this help message and exit
  --alignments-dir ALIGNMENTS_DIR, -i ALIGNMENTS_DIR
                        Path to the directory containing stockholm files
  --result-path RESULT_PATH
                        Path to create output directory RESULTS_AC/ under it
                        stores intermediate and final results
  --no-knn-vector       set this to avoid create an extra feature vector of
                        size 20bits necessary for KNN analysis
  --fasta-alignments    Set if alignments in alignments-dir are in fasta
                        alignment format
  --local-config LOCAL_CONFIG
                        if a local configuration is to be used, path to the
                        local configuration file
  --GC-config GC_CONFIG
                        if a custom GC(i.e. iterative clustering)
                        configuration is to be used, path to the GC
                        configuration file
  --sge-parallel        Run in parallel multicore/SGE mode
  --host-sge HOST_SGE   SGE host name, available options are: "docker",
                        "uibifr"
  --rep-fasta REP_FASTA
                        A fasta file containing one representative sequence
                        from each alignment under the input-directory.
                        Representative could be e.g. the human sequence of
                        each alignment
  --representative-keyword REPRESENTATIVE_KEYWORD
                        A keyword (as a substring of sequence ids header) that
                        can uniquely identify representative id of each
                        multiple alignment
  --temp-path TEMP_PATH
                        Path to store intermediate temporary direcotry named
                        TEMP_AC
  --structure-is-given  Do not use PETfold to predict the secondary consensus
                        " structure of each alignment
  --mfe                 Use MFE of each sequence as encoded graph, this option
                        ignores alignment and evolution information and is
                        mainly for evaluation purposes
  --path-graph          Use only path graph of each sequence as encoded graph,
                        this option ignores alignment and evolution
                        information and is mainly for evaluation purposes
  --path-abstract-graph
                        Use both path graph and abstract of each sequence as
                        encoded graph, this iscurrently for evaluation
                        purposes
  --only-representative
                        Only encode the given representative as graph and
                        ignore other sequences
  --max-selection-size MAX_SELECTION_SIZE
                        Max number of sequences, including representative, to
                        be encoded as graph size one is equivalent to only-
                        representative option
  --perform-evaluation  Perform knn/clustering evaluation after running the
                        pipeline
  --only-evaluation     Perform only knn/clustering evaluation on the
                        available results
  --threshold-reliability THRESHOLD_RELIABILITY
                        Petfold reliability threshold
  --predict-structure   Use PETfold to predict the secondary consensus
                        structure of each alignment
  --predict-alifold     Use RNAalifold to predict the secondary consensus
                        structure of each alignment
  --keep-temp, -k       Keep files after run finished
  --min-class-size MIN_CLASS_SIZE
                        Discard families where number of subalignments is less
                        than min
  --force, -f           Force overwriting to non-empty results-dir. WARNING:
                        Entire results-dir pass will be removed recursively!
  --skip-graph-encoding
                        Skip gspan graph generation and use current files
  --prefilter-blast     Prefilter input sequences using blastclust
  --encodeN-covarying   Extra special encoding for the covarying basepairs
  --covarying-Eval COVARYING_EVAL
                        Covarying basepairs with Evalue less than or equal
                        encoded
  --debug, -d           Debug mode, verbose information logged
  -job-id JOB_ID        dummy param

Example:

  python2 $(INSTALLATION_DIRECTORY)/bin/run_rnascclust.py --alignments-dir data/cliques-low/alignments/ --rep-fasta data/cliques-low/representatives/representatives.fa --predict-structure  --result-path . --local-config Config/docker-sge.config --sge'
```

## Contact
For questions, please contact us:

- Milad Miladi <miladim@cs.uni-freiburg.de>
- Alexander Junge <ajunge@rth.dk>