package MultipleAlignment::PETfold;
require Exporter;
use strict 'vars';
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw(read_stockholm
			);
#@EXPORT_OK = qw(			);


##############

#######################################################################
# Returns 
# INPUT: 
# OUTPUT: 
#######################################################################


my $STRUCT_PREFIX = "PETfold RNA structure:";


###############################################################################
# METHODS
###############################################################################
#write the struct to file
sub write_struct_to_file
{
	my ($file_name, $str) = @_;
	open (OUT_HANDLE, ">", $file_name);
	print OUT_HANDLE "$str\n";
}


## filter structure based on reliabilities
sub filter_structure_fullmatrix
{
	my ($struct, $matrix_ref, $threshold) = @_;
	my @struct_arr = split (//, $struct);
	
	(scalar(@struct_arr) == scalar(@{$matrix_ref})) or die "ERROR in filter_structure: structure and reliablities do not match:". scalar(@struct_arr) . "!=". scalar(@{$matrix_ref})."\n";
	my @filtered_struct_arr = split (//, $struct);

	
	#filter possibly halfly removed basepair, this should be resolved if above todo be done
	#check cons and filtered structure using stacks, so no crossing base pair is allowed
	my ( @open, @close ) = ();
	
	foreach my $idx (0..( @struct_arr-1 ) )
	{
		if ( $struct_arr[$idx] eq "(" ) {
			push( @open, $idx );
		}
		elsif ( $struct_arr[$idx] eq ")" ) {
			if (@open) {
				##either both ends should be remained or both should be filtered out 
				 my $left_idx  = pop(@open)  ;
				 #print "Reliabilty for bps ($idx, $left_idx) is " . $matrix_ref->[$idx][$left_idx]."\n";
				 if($matrix_ref->[$idx][$left_idx] < $threshold) {
				 	$filtered_struct_arr[$idx] = '.';
				 	$filtered_struct_arr[$left_idx] = '.';
				 }
				 
			}
			else {
				die("ERROR: The given cons structure has the wrong format, i.e. \n"
					  . "there are an uneven number of opening and closing brackets: $struct!\n"
				);
			}
		}
	}
	my $filtered_struct = join ('',@filtered_struct_arr);
	my $non_filtered_bp_count = $struct =~ tr/(//;
	my $filtered_bp_count = $filtered_struct =~ tr/(//;
	#print "#INFO: RelThreshold: $threshold non_filtered_bp_count: $non_filtered_bp_count filtered_bp_count: $filtered_bp_count\n";
	print "#INFO: RelThreshold: $threshold non_filtered_bp_count: $non_filtered_bp_count filtered_bp_count: $filtered_bp_count\n";

	print "Reliable bp filter is:\n" . join ('',@filtered_struct_arr) ."\n";

	return join ('',@filtered_struct_arr);
}

## filter structure based on reliabilities
sub filter_structure
{
	my ($struct, $reli, $threshold) = @_;
	my @struct_arr = split (//, $struct);
	my @rel_arr = split (/\s+/, $reli);
	(@struct_arr == @rel_arr) or die "ERROR in filter_structure: structure and reliablities do not match\n";
	my @filtered_struct_arr  ;
	foreach my $idx (0..( @rel_arr-1 ) )
	{
#		print "$idx: $rel_arr[$idx]  $struct_arr[$idx]\n";
		
	#todo: WARNING filtering should be based on a basepair on relibility of individual base positions
		if ( (1-$rel_arr[$idx]) <  $threshold  && ($struct_arr[$idx] =~ /[\(\)]/))
		{
			# base has is lower than threshold and should be filterd out
			$filtered_struct_arr[$idx] = ".";
		}
		else
		{
			$filtered_struct_arr[$idx] = $struct_arr[$idx];
#			if ($struct_arr[$idx] =~ /[\(\)]/) {
#				print $rel_arr[$idx] . " < " .  $threshold  . "--" ;
#			}
		}
#		print "\n";
	}
	

	
	#filter possibly halfly removed basepair, this should be resolved if above todo be done
	#check cons and filtered structure using stacks, so no crossing base pair is allowed
	my ( @open, @close ) = ();
	
	foreach my $idx (0..( @struct_arr-1 ) )
	{
		if ( $struct_arr[$idx] eq "(" ) {
			push( @open, $idx );
		}
		elsif ( $struct_arr[$idx] eq ")" ) {
			if (@open) {
				##either both ends should be remained or both should be filtered out 
				 my $left_idx  = pop(@open)  ;
				 if ( $filtered_struct_arr[$left_idx] ne "(" 
				 		or  $filtered_struct_arr[$idx] ne ")")
		 		{
		 			## make sure both basepair ends are removed 
		 			$filtered_struct_arr[$left_idx] = ".";
		 			$filtered_struct_arr[$idx] = ".";
		 		}
			}
			else {
				die("ERROR: The given cons structure has the wrong format, i.e. \n"
					  . "there are an uneven number of opening and closing brackets: $struct!\n"
				);
			}
		}
	}
	my $filtered_struct = join ('',@filtered_struct_arr);
	my $non_filtered_bp_count = $struct =~ tr/(//;
	my $filtered_bp_count = $filtered_struct =~ tr/(//;
	print "#INFO: RelThreshold: $threshold non_filtered_bp_count: $non_filtered_bp_count filtered_bp_count: $filtered_bp_count\n";


	print "Reliable filter is:\n" . join ('',@filtered_struct_arr) ."\n";

	return join ('',@filtered_struct_arr);
}

##parse  output log (i.e. redirected stdout ) of PETfold
sub parse_outlog
{
	my ($log_file) = @_;
	open (my $in_handle, "<", $log_file) or die "Failed to open: $!";

	my $cons_struct = "";
	# Reading from that filehandle
	while(my $line = <$in_handle>) {
		if ($line =~ /^$STRUCT_PREFIX\s+(.+)/)
		{
			$cons_struct = $1;
			chomp($cons_struct);
			print "cons struct found:\n$cons_struct\n";
			return $cons_struct;
		}
	}	
	
	die "ERROR parse_PETfold_outlog: no cons struct found in $log_file\n";
}

sub matrix_read_file {
    my ($filename) = @_;
    open (FIN, $filename) || die "Could not open $filename: $!";
	my $firstLine = <FIN>;
	my @retMatrix = ();
	chomp($firstLine);
	my $mat_size = int($firstLine);
	my $rowCounter = 0;
	print "MATszie is $mat_size\n";
    while (my $line = <FIN>) {
        chomp($line);
        next if $line =~ /^\s*$/; # skip blank lines
        $rowCounter += 1;
                   
		my (@row) = split (/\s+/, $line);
		            
		push (@retMatrix, \@row); 
		# insert the row-array into
                                           # the outer matrix array
    }
    close(FIN);
    return \@retMatrix;
}

##parse reliability pp file generated by PETfold
sub parse_reali_pp
{
	my ($pp_file) = @_;
	open (my $in_handle, "<", $pp_file) or die "Failed to open: $!";
	my $line_count = 0;
	my $cons_rels = "";

	my $firstLine = <$in_handle>;
	my @retMatrix = ();
	chomp($firstLine);
	my $mat_size = int($firstLine);
	my $rowCounter = 0;

	# Reading from that filehandle
	while(defined(my $line = <$in_handle>) and $rowCounter < $mat_size) {
		chomp($line);
        next if $line =~ /^\s*$/; # skip blank lines
        $rowCounter += 1;
                   
		my (@row) = split (/\s+/, $line);
		            
		push (@retMatrix, \@row); 
	        
	}
	my $last_line = "";
	while(my $line = <$in_handle>) {
		chomp($line);
        next if $line =~ /^\s*$/; # skip blank lines
		$last_line = $line;
	}

   	# last line is consensus reliablity?
	$cons_rels = $last_line;

	($cons_rels ne "" ) or die "consensus reliabilities could not be read from $pp_file\n";
	(scalar(@retMatrix) == $mat_size)  or die("Mismtach between expected and read matrix sizes: $mat_size != ".scalar(@retMatrix)."\n");
	#print STDOUT "Read " . scalar(@retMatrix) . " matrix rows from $pp_file\n";
	return ($cons_rels, $mat_size, \@retMatrix);
}


######################################################################
# Parse PETfold outfold and return struture (i.e. first line) 
# INPUT: Logfile
# OUTPUT: 
#######################################################################
sub parse_petfold_cons_file
{
	my ($file_name) = @_;
	open (IN_HANDLE, "<", $file_name);
	my $line = <IN_HANDLE>;
	chomp ($line);
	return $line;	
}


