package MultipleAlignment::alifold;
require Exporter;
use strict 'vars';
use warnings;

our @ISA = qw(Exporter);
#our @EXPORT = qw(parse_alifold_outlog_get_covarying_bps,
#	parse_rscape_outlog_get_covarying_bps
#			);
#@EXPORT_OK = qw(			);


##############

#######################################################################
# Returns 
# INPUT: 
# OUTPUT: 
#######################################################################


my $STRUCT_PREFIX = "PETfold RNA structure:";


###############################################################################
# METHODS
###############################################################################
##parse  output log (i.e. redirected stdout ) of PETfold
sub parse_rscape_outlog_get_covarying_bps
{
	my ($log_file, $ref_struct, $threshold, $max_coverage_ratio, $min_coverage_ratio) = @_;
	print "$log_file\n";
	open (my $in_handle, "<", $log_file) or die "Failed to open: $!";

	my @cons_struct_arr = ('.') x length($ref_struct); 
	my $basepairs_href = get_basepair_strings($ref_struct);
	my %covaring_positions = ();
	my $change_count = 0.0;
	# Reading from that filehandle
	while(my $line = <$in_handle>) {
		if ($line =~ /^#/)
		{
			next;
		}
		chomp($line);
		if ($line eq '') {
			next;
		}
		# type left_pos       right_pos        score   E-value
		# *     3              46               6.79    0.000143788
		my @splits = split ' ', $line;
		die "split cov-bon failed at line: $line" if ( not ((scalar @splits) == 5 or (scalar @splits) == 4));
		#print "splits @splits\n";
		my ($type, $i, $j, $score, $eval); 
		if ( @splits == 5){
			($type, $i, $j, $score, $eval) = ($splits[0], ($splits[1]) , ($splits[2]), 
			                                        ($splits[3]), ($splits[4]));
			
		}
		else{
			$type = '';
			($i, $j, $score, $eval) = (($splits[0]), ($splits[1]), ($splits[2]), ($splits[3]));
		}
		#print "r-scape: <$type>, $i,$j, $score, $eval\n";
		
		if (exists $basepairs_href->{"$i,$j"}) {
			die "expecting matchin basepair with * type, received type:$type: line$line\n" if ($type ne '*');
			#print "Matching: ! ".  $i. " ". $j. "\n";
			$cons_struct_arr[$i-1] = '(';
			$cons_struct_arr[$j-1] = ')';
			$change_count += 2.0;
			if ($change_count/length($ref_struct) > $max_coverage_ratio) {
				print "parse_rscape_outlog_get_covarying_bps: Max coverage ratio reached\n";
				last;
			}
#			if ($threshold < $first_split[1]) {
#				print "Above threshold: " . $first_split[0]. " ". $first_split[1]. "\n";
#				$cons_struct_arr[$ij[0]-1] = '(';
#				$cons_struct_arr[$ij[1]-1] = ')';
#			}
		}
	
	}
	close $in_handle;
	print "$change_count/".((scalar keys %{$basepairs_href})*2)."=". ($change_count/(scalar keys %{$basepairs_href})) . "\n";
	if ($change_count/((scalar keys %{$basepairs_href})*2) < $min_coverage_ratio) {
		print "parse_rscape_outlog_get_covarying_bps: Min coverage ratio not reached\n";
		return '';
	}
	return join('', @cons_struct_arr);

}

sub parse_alifold_outlog_get_consensus
{
	my ($log_file) = @_;
	open (my $in_handle, "<", $log_file) or die "Failed to open: $!";

	my $cov_section = 0; 
	my $last_line = "";
	# Reading from that filehandle
	while(my $line = <$in_handle>) {
		if ($line =~ /^#BEGIN covariation scores/)
		{
			
		}
		elsif ($line =~ /^#END covariation scores/) {
			
			$cov_section = 2;
		}
		elsif ($cov_section == 1)
		{
			next;
		}
		else {
			chomp($line);
			if ($line eq '') {
				next;
			}
			$last_line = $line;
		}
		
		
		
	}	
	print "alifold: lastline: $last_line\n";
	close $in_handle;
	my @lastsplits = split ' ', $last_line;
	die "split lastline failed at line: $last_line" if ( (scalar @lastsplits) < 2);
	return $lastsplits[0];

}
##parse  output log (i.e. redirected stdout ) of PETfold
sub parse_alifold_outlog_get_covarying_bps
{
	my ($log_file, $ref_struct, $threshold) = @_;
	open (my $in_handle, "<", $log_file) or die "Failed to open: $!";

	my @cons_struct_arr = ('.') x length($ref_struct);
	my $cov_section = 0; 
	my $basepairs_href = get_basepair_strings($ref_struct);
	my %covaring_positions = ();
	# Reading from that filehandle
	while(my $line = <$in_handle>) {
		if ($line =~ /^#BEGIN covariation scores/)
		{
			$cov_section = 1;
		}
		elsif ($line =~ /^#END covariation scores/) {
			close $in_handle;
			
			return join('', @cons_struct_arr);
		}
		elsif ($cov_section == 1)
		{
			chomp($line);
			if ($line eq '') {
				next;
			}
			
			my @first_split = split ' ', $line;
			die "split cov-bon failed at line: $line" if ( (scalar @first_split) != 2);
			my @ij = split ',', $first_split[0];
			#print "cov-bonus: " . $ij[0].  "," . $ij[1] . ": ". $first_split[1] . "\n";
			
			if (exists $basepairs_href->{$first_split[0]}) {
				#print "Matching: ! ".  $first_split[0]. " ". $first_split[1]. "\n";
				if ($threshold < $first_split[1]) {
					print "Above threshold: " . $first_split[0]. " ". $first_split[1]. "\n";
					$cons_struct_arr[$ij[0]-1] = '(';
					$cons_struct_arr[$ij[1]-1] = ')';
				}
			}
		}
	}	
	
	die "ERROR parse_outlog_get_covarying_bps: no cons struct found in $log_file\n";
}

sub get_basepair_strings
{
	#print "get_basepair_strings:\n";
	my ($struct) = @_;
	my @struct_arr = split (//, $struct);
	my %basepairs = ();
	my ( @open, @close ) = ();
	
	foreach my $idx (0..( @struct_arr-1 ) )
	{
		if ( $struct_arr[$idx] eq "(" ) {
			push( @open, $idx );
		}
		elsif ( $struct_arr[$idx] eq ")" ) {
			if (@open) {
				##either both ends should be remained or both should be filtered out 
				 my $left_idx  = pop(@open)  ;
				 my $bpstr = sprintf ("%d,%d",($left_idx+1),($idx+1));
				 #print $bpstr."\n";
				 $basepairs{$bpstr} = 1;
			}
			else {
				die("ERROR: The given cons structure has the wrong format, i.e. \n"
					  . "there are an uneven number of opening and closing brackets: $struct!\n"
				);
			}
		}
	}
	return \%basepairs;
}


